import { world, system, BlockPermutation } from "@minecraft/server"

world.afterEvents.entityHurt.subscribe(ev => {
    const { hurtEntity } = ev

    if (hurtEntity.typeId == 'minecraft:iron_golem') {
        const villagerHealerEntities = hurtEntity.dimension.getEntities({ location: hurtEntity.location, maxDistance: 25, type: "eu:villager_golem_healer" })
        villagerHealerEntities.forEach(villagerHealerEntity => {
            if (!villagerHealerEntity.isValid) { return }
            if (villagerHealerEntity.getProperty('eu:is_crafting_golem') || villagerHealerEntity.getProperty('eu:is_admiring_poppy')) { return }

            villagerHealerEntity.triggerEvent("eu:start_crafting_golem")
        })

    }
})


function findAndDestroyFirstPoppy(center, dimension, entity) {
    for (let dx = -1; dx <= 1; dx++) {
        for (let dz = -1; dz <= 1; dz++) {
            const checkLocation = {
                x: center.x + dx,
                y: center.y,
                z: center.z + dz
            };

            const block = dimension.getBlock(checkLocation);
            if (block && block.typeId == 'minecraft:poppy') {
                dimension.runCommand(`setblock ${checkLocation.x} ${checkLocation.y} ${checkLocation.z} air destroy`);
                entity.playAnimation('animation.villager_golem_healer.poppy');
                return;
            }
        }
    }
}

world.afterEvents.dataDrivenEntityTrigger.subscribe(ev => {
    const { entity, eventId } = ev

    if (entity.typeId == 'minecraft:iron_golem' && eventId == 'minecraft:from_village') {
        const { dimension, location } = entity
        dimension.spawnEntity('eu:villager_golem_healer', location)
    }
    if (entity.typeId == 'eu:mini_golem' && eventId == 'eu:found_golem') {
        const { dimension, location } = entity
        const golemEntity = entity.dimension.getEntities({ location: location, closest: 1, families: ["irongolem"], excludeTypes: ['eu:mini_golem'] })[0]
        const healthComp = golemEntity.getComponent('health')
        healthComp.setCurrentValue(healthComp.currentValue + 15)
        dimension.playSound('smithing_table.use', location)
        entity.remove()
        //world.sendMessage('hola golem')

    }

    if (entity.typeId == 'eu:villager_golem_healer') {
        const { dimension, location } = entity
        if (eventId == 'eu:start_crafting_golem') {
            system.runTimeout(() => {
                const view = entity.getViewDirection()
                const blockAbove = dimension.getTopmostBlock({ x: location.x + view.x * 2, z: location.z + view.z * 2 })
                dimension.spawnEntity('eu:mini_golem', { x: blockAbove.location.x, y: blockAbove.location.y + 1, z: blockAbove.location.z })
            }, 3.5 * 20)
        } else if (eventId == 'eu:found_poppy') {
            entity.addEffect('regeneration', 20 * 10)
            findAndDestroyFirstPoppy(location, dimension, entity);
        }
    }
})

/* world.afterEvents.entitySpawn.subscribe(ev => {
    const { entity } = ev
    if (entity.typeId == 'eu:mini_golem') {

        entity.playAnimation('animation.mini_golem.attack')


    }
}) */